/*******************************************************************************
 * Copyright (c) 2013 THALES GLOBAL SERVICES.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 *******************************************************************************/
package org.eclipse.sirius.diagram.business.internal.query;

import org.eclipse.sirius.diagram.ContainerLayout;
import org.eclipse.sirius.diagram.description.ContainerMapping;

/**
 * A class aggregating all the queries (read-only!) having a ContainerMapping as
 * a starting point.
 * 
 * @author mporhel
 * 
 */
public class ContainerMappingExperimentalQuery {

    private ContainerMapping mapping;

    /**
     * Create a new query.
     * 
     * @param mapping
     *            the element to query.
     */
    public ContainerMappingExperimentalQuery(ContainerMapping mapping) {
        this.mapping = mapping;
    }

    /**
     * Tests whether or not the {@link ContainerMapping} is a vertical stack
     * container, according to the {@link ContainerLayout} specified in the VSM.
     * 
     * @return <code>true</code> if the {@link ContainerMapping} has a vertical
     *         stack children presentation.
     */
    public boolean isVerticalStackContainer() {
        return ContainerLayout.VERTICAL_STACK == mapping.getChildrenPresentation();
    }
    
    /**
     * Tests whether or not the {@link ContainerMapping} is a horizontal stack
     * container, according to the {@link ContainerLayout} specified in the VSM.
     * 
     * @return <code>true</code> if the {@link ContainerMapping} has a horizontal
     *         stack children presentation.
     */
    public boolean isHorizontalStackContainer() {
        return ContainerLayout.HORIZONTAL_STACK == mapping.getChildrenPresentation();
    }

    /**
     * Tests whether or not the {@link ContainerMapping} is a region container,
     * according to the {@link ContainerLayout} specified in the VSM.
     * 
     * @return <code>true</code> if the {@link ContainerMapping} is a region
     *         container.
     */
    public boolean isRegionContainer() {
        return isVerticalStackContainer() || isHorizontalStackContainer();
    }

    /**
     * Tests whether or not the {@link ContainerMapping} is a region (e.g. child
     * of a region container, see {@link ContainerMappingExperimentalQuery#isRegionContainer()}).
     * 
     * @return <code>true</code> if the {@link ContainerMapping} is a region.
     */
    public boolean isRegion() {
        if (mapping != null && mapping.eContainer() instanceof ContainerMapping) {
            return new ContainerMappingExperimentalQuery((ContainerMapping) mapping.eContainer()).isRegionContainer();
        }
        return false;
    }

}
